#include "mqtt.hpp"


void Mymqtt::begin(const char * server, uint16_t mqttPort, const char* mqttUser, const char* mqttPassword)
{
    _server = server;
    _port = mqttPort;
    _mqttPassword = mqttPassword;
    _mqttUser = mqttUser;
    espClient = new WiFiClientSecure();//new WiFiClient();
    client = new PubSubClient(*espClient);
    client->setBufferSize(MQTT_BUFFER_SIZE);
}

void callback(char* topic, byte* payload, unsigned int length) {
  String incommingMessage = "";
  for (int i = 0; i < length; i++) incommingMessage+=(char)payload[i];
  Serial.println("Message arrived ["+String(topic)+"]"+incommingMessage);
  // check for other commands
  /* else if( strcmp(topic,command2_topic) == 0){
  if (incommingMessage.equals(“1”)) { } // do something else
  }
  */
}

bool Mymqtt::connect()
{
    static unsigned long last_connection_try_ms = 0;
    unsigned long start_ms = 0;
    if(true == is_connected())
    {
        return true;
    }
    if((last_connection_try_ms == 0) || (millis() - last_connection_try_ms > MQTT_TRY_TO_CONNECT_MS))
    {
      last_connection_try_ms = millis();
      if (WiFi.status() == WL_CONNECTED)
      {
          Serial.printf("(Re)connecting to MQTT ... server '%s' port: %d \n", _server.c_str(), _port);
          espClient->setCACert(root_ca);
          client->setServer(_server.c_str(), _port);
          client->setCallback(callback);
          start_ms = millis();
          String clientId = MQTT_CHANNEL "-"; // Create a random client ID
          clientId += String(random(0xffff), HEX);
          if (client->connect(MQTT_CHANNEL, _mqttUser.c_str(), _mqttPassword.c_str()))
          {
              Serial.println(" -> MQTT connected");
              String connect_msg = "Connected from Infinitymachine of MakerLab Murnau e.V at " + String(millis());
              if(client->publish("infinity/connected", connect_msg.c_str()))
              {
                Serial.println("Connection and initial publish passed.");
              }
              else
              {
                Serial.println("Connection and initial publish FAILED.");
              }
          } 
          else 
          {
              Serial.print(" -> MQTT connection failed with state ");
              Serial.println(client->state());
          } 
      }
      else
      {
          Serial.println("Not connected to the WiFi network");
      }
    }
    return(is_connected());
}

bool Mymqtt::is_connected()
{
    return(client->connected());
}

bool Mymqtt::publish(String topic,String data)
{
  if(true == connect())
  {
    client->publish(topic.c_str(), data.c_str(), true);
    return(true);
  }
  return(false);
}

bool Mymqtt::loop()
{
  if(true == connect())
  {
    client->loop();
    return(true);
  }
  return(false);
}

bool Mymqtt::send_and_loop(String topic,String data)
{
  static unsigned long _last_sent_ms = 0;
  if(millis() - _last_sent_ms >= SEND_MQTT_EVERY_MS)
  {
    _last_sent_ms = millis();
    if(true == connect())
    {
      client->loop();
      if(data != _data_sent)
      {
        if((MQTT_BUFFER_SIZE - 20) > data.length())
        {
          if( client->publish(topic.c_str(), data.c_str() , data.length()) )
          {
            _data_sent = data;
          }
          else
          {
            //Serial.printf("Sending data (topic: %s): FAILED!!!\n -> '%s'\n", topic.c_str(), data.c_str());
            return(false);
          }
        }
        else
        {
          //Serial.printf("Data length (%d) is larger than MQTT buffer(%d + 20bytes header)\n", data.length(), MQTT_BUFFER_SIZE);
          return(false);
        }
      }
      return(true);
    }
  }
  return(false);
}
